@@ -9,6 +9,7 @@ use warnings;
 use utf8;
 use parent 'Dallycot::AST';
 
+use Carp qw(croak);
 use List::MoreUtils qw(all_u);
 
 use Readonly;
@@ -46,16 +47,33 @@ sub execute {
 
   return $engine->collect( map { [ $_, $NUMERIC ] } @$self )->then(
     sub {
-      if( all_u { $_ -> isa('Dallycot::Value::Duration') } @_ ) {
+      my $num_durations = grep { $_ -> isa('Dallycot::Value::Duration') } @_;
+      my $num_dates = grep { $_ -> isa('Dallycot::Value::DateTime') } @_;
+      if( $num_durations == @_ ) {
         my (@values) = map { $_ -> value } @_;
-        my $acc = ( pop @values ) -> clone;
+        my $first = pop @values;
+        my $next = pop @values;
+        my $acc = $first + $next;
         $acc += $_ for @values;
         return Dallycot::Value::Duration->new( object => $acc );
       }
+      elsif( $num_durations + $num_dates == @_ && $num_dates == 1 ) {
+        my (@values) = map { $_ -> value } @_;
+        my $first = shift @values;
+        my $next = shift @values;
+        my $acc = $first + $next;
+        $acc += $_ for @values;
+        return Dallycot::Value::DateTime->new( object => $acc );
+      }
+      elsif( $num_dates > 1 ) {
+        croak 'Multiple dates can not be added together';
+      }
       else {
         my (@values) = map { $_->value } @_;
 
-        my $acc = ( pop @values )->copy;
+        my $acc = pop @values;
+        my $next = pop @values;
+        $acc = $acc + $next;
 
         $acc += $_ for @values;
 
